//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclIrDA"
#pragma link "wclIrDAClients"
#pragma link "wclIrDADevices"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btDiscoverClick(TObject *Sender)
{
  lvDevices->Items->Clear();

  TwclIrDADevices Devices;
  int Res = wclIrDAReceiver->Discover(Devices);
  try
  {
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
    else
      if (Devices.Length == 0)
        MessageDlg("No devices were found.", mtInformation, TMsgDlgButtons() << mbOK, 0);
      else
        for (int i = 0; i < Devices.Length; i++)
        {
          TListItem* Item = lvDevices->Items->Add();
          Item->Caption = IntToHex((int)Devices[i].Address, 8);
          Item->SubItems->Add(Devices[i].Name);
        }
  }
  __finally
  {
    Devices.Length = 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbEvents->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btDisconnectClick(TObject *Sender)
{
  int Res = wclIrDAClient->Disconnect();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btConnectClick(TObject *Sender)
{
  if (lvDevices->Selected == NULL)
    MessageDlg("Select device", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    wclIrDAClient->Address = StrToInt("$" + lvDevices->Selected->Caption);
    wclIrDAClient->Timeout = StrToInt(edTimeout->Text);

    int Res = wclIrDAClient->Connect();
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSendClick(TObject *Sender)
{
  AnsiString Ansi = AnsiString(edText->Text);
  unsigned int Sent;
  int Res = wclIrDAClient->Write(Ansi.c_str(), Ansi.Length(), Sent);
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
  else
    MessageDlg("Sent: " + IntToStr((int)Sent) + " from " +
      IntToStr(Ansi.Length()), mtInformation, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAClientConnect(TObject *Sender, const int Error)
{
  if (Error == WCL_E_SUCCESS)
  {
    lbEvents->Items->Add("Connected");

    // Try to get read and write buffers size.
    unsigned int Size = 0;

    int Res = wclIrDAClient->GetReadBufferSize(Size);
    if (Res != WCL_E_SUCCESS)
      lbEvents->Items->Add("Get read buffer error: 0x" + IntToHex(Res, 8));
    else
      edReadBuffer->Text = IntToStr((int)Size);

    Size = 0;
    Res = wclIrDAClient->GetWriteBufferSize(Size);
    if (Res != WCL_E_SUCCESS)
      lbEvents->Items->Add("Get write buffer error: 0x" + IntToHex(Res, 8));
    else
      edWriteBuffer->Text = IntToStr((int)Size);
  }
  else
    lbEvents->Items->Add("Connect error: 0x" + IntToHex(Error, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAClientDisconnect(TObject *Sender,
      const int Reason)
{
  lbEvents->Items->Add("Disconnected: 0x" + IntToHex(Reason, 8));

  edReadBuffer->Text = "0";
  edWriteBuffer->Text = "0";
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAClientData(TObject *Sender, const Pointer Data,
      const DWORD Size)
{
  if (Size > 0)
  {
    AnsiString Str;
    Str.SetLength(Size);
    CopyMemory(Str.c_str(), Data, Size);

    lbEvents->Items->Add("Received: " + Str);
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  wclIrDAClient->Disconnect();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSetBuffersClick(TObject *Sender)
{
  // Try to set read buffer size first.
  unsigned int Size = StrToInt(edReadBuffer->Text);
  int Res = wclIrDAClient->SetReadBufferSize(Size);
  if (Res == WCL_E_SUCCESS)
    lbEvents->Items->Add("Read buffer size changed.");
  else
    lbEvents->Items->Add("Set read buffer size error: 0x" + IntToHex(Res, 8));

  // Now write buffer.
  Size = StrToInt(edWriteBuffer->Text);
  Res = wclIrDAClient->SetWriteBufferSize(Size);
  if (Res == WCL_E_SUCCESS)
    lbEvents->Items->Add("Write buffer size changed.");
  else
    lbEvents->Items->Add("Set write buffer size error: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------

