//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclIrDA"
#pragma link "wclIrDADevices"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  cbClassName->ItemIndex = 0;
  cbClassNameChange(cbClassName);

  FPowerMonitor = new TwclPowerEventsMonitor();
  FPowerMonitor->OnPowerStateChanged = PowerStateChanged;
  FPowerMonitor->Open();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btDiscoverClick(TObject *Sender)
{
  lvDevices->Items->Clear();

  TwclIrDADevices Devices;
  int Res = wclIrDAReceiver->Discover(Devices);
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
  else
  {
    if (Devices.Length == 0)
      MessageDlg("Nothing found", mtWarning, TMsgDlgButtons() << mbOK, 0);
    else
    {
      for (int i = 0; i < Devices.Length; i++)
      {
        TwclIrDADevice Device = Devices[i];

        TListItem* Item = lvDevices->Items->Add();
        Item->Caption = IntToHex((int)Device.Address, 8);
        Item->SubItems->Add(Device.Name);

        String Str = "";
        TwclIrDAHint j = ihPnp;
        while (j <= ihObex)
        {
          if (Device.Hints.Contains(j))
          {
            switch (j)
            {
              case ihPnp:
                Str = Str + "ihPnp ";
                break;
              case ihPda:
                Str = Str + "ihPda ";
                break;
              case ihComputer:
                Str = Str + "ihComputer ";
                break;
              case ihPrinter:
                Str = Str + "ihPrinter ";
                break;
              case ihModem:
                Str = Str + "ihModem ";
                break;
              case ihFax:
                Str = Str + "ihFax ";
                break;
              case ihLan:
                Str = Str + "ihLan ";
                break;
              case ihTelephony:
                Str = Str + "ihTelephony ";
                break;
              case ihFileServer:
                Str = Str + "ihFileServer ";
                break;
              case ihComm:
                Str = Str + "ihComm ";
                break;
              case ihMessage:
                Str = Str + "ihMessage ";
                break;
              case ihHttp:
                Str = Str + "ihHttp ";
                break;
              case ihObex:
                Str = Str + "ihObex ";
                break;
              default:
                Str = Str + "UNKNOWN ";
                break;
            }
          }
          j = (TwclIrDAHint)((unsigned char)j + 1);
        }
        Item->SubItems->Add(Str);

        switch (Device.CharSet)
        {
          case csAscii:
            Item->SubItems->Add("csAscii");
            break;
          case csIso8859_1:
            Item->SubItems->Add("csIso8859_1");
            break;
          case csIso8859_2:
            Item->SubItems->Add("csIso8859_2");
            break;
          case csIso8859_3:
            Item->SubItems->Add("csIso8859_3");
            break;
          case csIso8859_4:
            Item->SubItems->Add("csIso8859_4");
            break;
          case csIso8859_5:
            Item->SubItems->Add("csIso8859_5");
            break;
          case csIso8859_6:
            Item->SubItems->Add("csIso8859_6");
            break;
          case csIso8859_7:
            Item->SubItems->Add("csIso8859_7");
            break;
          case csIso8859_8:
            Item->SubItems->Add("csIso8859_8");
            break;
          case csIso8859_9:
            Item->SubItems->Add("csIso8859_9");
            break;
          case csUnicode:
            Item->SubItems->Add("csUnicode");
            break;
          default:
            Item->SubItems->Add("UNKNOWN");
            break;
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::ntSubscribeClick(TObject *Sender)
{
  int Res = wclIrDAReceiver->Subscribe();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btUnsubscribeClick(TObject *Sender)
{
  int Res = wclIrDAReceiver->Unsubscribe();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbEvents->Items->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::cbClassNameChange(TObject *Sender)
{
  cbAttributeName->Items->Clear();

  switch (cbClassName->ItemIndex)
  {
    case 0:
         cbAttributeName->Items->Add("DeviceName");
         cbAttributeName->Items->Add("IrLMPSupport");
         break;
    case 1:
         cbAttributeName->Items->Add("Parameters");
         cbAttributeName->Items->Add("IrDA:IrLMP:LsapSel");
         cbAttributeName->Items->Add("IrDA:TinyTP:LsapSel");
         cbAttributeName->Items->Add("IrDA:InstanceName");
         break;
    case 2:
      cbAttributeName->Items->Add("IrDA:TinyTP:LsapSel");
      break;
    case 3:
      cbAttributeName->Items->Add("IrDA:IrLMP:LsapSel");
      break;
    case 4:
      cbAttributeName->Items->Add("Integer value");
      cbAttributeName->Items->Add("Octet Sequence");
      cbAttributeName->Items->Add("User string");
      break;
  }

  cbAttributeName->ItemIndex = 0;
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btQueryClick(TObject *Sender)
{
  if (lvDevices->Selected == NULL)
    MessageDlg("Select device", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    TwclIrDAAttribute Attrib;
    int Res = wclIrDAReceiver->QueryAttribute(
      StrToInt("$" + lvDevices->Selected->Caption),
      cbClassName->Items->Strings[cbClassName->ItemIndex],
      cbAttributeName->Items->Strings[cbAttributeName->ItemIndex], Attrib);
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
    else
    {
      switch (Attrib.ValueType)
      {
        case atInteger:
          MessageDlg("Integer: " + IntToStr(Attrib.IntegerValue),
            mtInformation, TMsgDlgButtons() << mbOK, 0);
          break;

        case atSequence:
          {
            String Str = "";
            if (Attrib.SequenceValue.Length != 0)
            {
              if (Attrib.SequenceValue.Length > 0)
                for (int i = 0; i < Attrib.SequenceValue.Length; i++)
                  Str = Str + IntToHex(Attrib.SequenceValue[i], 2);

              Attrib.SequenceValue.set_length(0);
            }

            MessageDlg("Sequence: " + Str, mtInformation, TMsgDlgButtons() << mbOK, 0);
            break;
          }

        case atString:
          MessageDlg("String: " + Attrib.StringValue, mtInformation, TMsgDlgButtons() << mbOK, 0);
          break;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btStartLazyDiscoveringClick(TObject *Sender)
{
  int Res = wclIrDAReceiver->StartLazyDiscovering();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btStopLazyDiscoveringClick(TObject *Sender)
{
  int Res = wclIrDAReceiver->StopLazyDiscovering();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAReceiverChange(TObject *Sender)
{
  if (wclIrDAReceiver->Available)
    lbEvents->Items->Add("Hardware changed. IrDA available.");
  else
    lbEvents->Items->Add("Hardware changed. IrDA NOT available.");
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAReceiverDevicesFound(TObject *Sender,
      const TwclIrDADevice &Device)
{
  lbEvents->Items->Add("Device found: " +
    IntToHex((int)Device.Address, 8) + "; " + Device.Name);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  wclIrDAReceiver->StopLazyDiscovering();
  wclIrDAReceiver->Unsubscribe();

  FPowerMonitor->Close();
  FPowerMonitor->Free();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::PowerStateChanged(TObject* Sender,
  const TwclPowerState State)
{
  switch (State)
  {
    case psResumeAutomatic:
      lbEvents->Items->Add("Power state: psResumeAutomatic");
      break;

    case psResume:
      lbEvents->Items->Add("Power state: psResume");
      break;

    case psSuspend:
      wclIrDAReceiver->StopLazyDiscovering();
      wclIrDAReceiver->Unsubscribe();

      lbEvents->Items->Add("Power state: psSuspend");
      break;

    case psUnknown:
      lbEvents->Items->Add("Power state: psUnknown");
      break;
  }
}
//---------------------------------------------------------------------------
