//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclIrDA"
#pragma link "wclIrDAServers"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btListenClick(TObject *Sender)
{
  int Res = wclIrDAServer->Listen();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btCloseClick(TObject *Sender)
{
  int Res = wclIrDAServer->Close();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbEvents->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btDisconnectClick(TObject *Sender)
{
  if (lvClients->Selected == NULL)
    MessageDlg("Select client", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    int Res = ((TwclIrDAServerClientConnection*)lvClients->Selected->Data)->Disconnect();
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSendClick(TObject *Sender)
{
  if (lvClients->Selected == NULL)
    MessageDlg("Select client", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    AnsiString Ansi = AnsiString(edText->Text);
    unsigned int Sent;
    int Res = ((TwclIrDAServerClientConnection*)lvClients->Selected->Data)->Write(Ansi.c_str(),
      Ansi.Length(), Sent);
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
    else
      MessageDlg("Sent: " + IntToStr((int)Sent) + " from " + IntToStr(Ansi.Length()),
        mtInformation, TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------


void __fastcall TfmMain::wclIrDAServerListen(TObject *Sender)
{
  lbEvents->Items->Add("Server listening");
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  wclIrDAServer->Close();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btGetBuffersClick(TObject *Sender)
{
  if (lvClients->Selected == NULL)
    MessageDlg("Select client", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    TwclIrDAServerClientConnection* Client = (TwclIrDAServerClientConnection*)lvClients->Selected->Data;

    // Get read buffer.
    unsigned int Size = 0;
    int Res = Client->GetReadBufferSize(Size);
    if (Res != WCL_E_SUCCESS)
      lbEvents->Items->Add("Get read buffer error: 0x" + IntToHex(Res, 8));
    else
      edReadBuffer->Text = IntToStr((int)Size);

    // Get write buffer.
    Size = 0;
    Res = Client->GetWriteBufferSize(Size);
    if (Res != WCL_E_SUCCESS)
      lbEvents->Items->Add("Get write buffer error: 0x" + IntToHex(Res, 8));
    else
      edWriteBuffer->Text = IntToStr((int)Size);
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSetBuffersClick(TObject *Sender)
{
  if (lvClients->Selected == NULL)
    MessageDlg("Select client", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    TwclIrDAServerClientConnection* Client = (TwclIrDAServerClientConnection*)lvClients->Selected->Data;

    // Set read buffer.
    unsigned int Size = StrToInt(edReadBuffer->Text);
    int Res = Client->SetReadBufferSize(Size);
    if (Res != WCL_E_SUCCESS)
      lbEvents->Items->Add("Set read buffer error: 0x" + IntToHex(Res, 8));

    // Set write buffer.
    Size = StrToInt(edWriteBuffer->Text);
    Res = Client->SetWriteBufferSize(Size);
    if (Res != WCL_E_SUCCESS)
      lbEvents->Items->Add("Set write buffer error: 0x" + IntToHex(Res, 8));
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btAddIasRecordClick(TObject *Sender)
{
  switch (cbIasValueType->ItemIndex)
  {
    case 0:
      {
        int Res = wclIrDAServer->AddIasRecord("IrDAFramework", "Integer value",
          1234567);
        if (Res != WCL_E_SUCCESS)
          lbEvents->Items->Add("Add IAS record error: 0x" + IntToHex(Res, 8));
        break;
      }

    case 1:
      {
        unsigned char Arr[8] = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
          0x07, 0x08 };
        int Res = wclIrDAServer->AddIasRecord("IrDAFramework", "Octet Sequence",
          Arr, 8);
        if (Res != WCL_E_SUCCESS)
          lbEvents->Items->Add("Add IAS record error: 0x" + IntToHex(Res, 8));
        break;
      }

    case 2:
      {
        unsigned char Arr[11] = { 85, 115, 101, 114, 32, 115, 116, 114, 105,
          110, 103 };
        int Res = wclIrDAServer->AddIasRecord("IrDAFramework", "User string",
          csAscii, Arr, 11);
        if (Res != WCL_E_SUCCESS)
          lbEvents->Items->Add("Add IAS record error: 0x" + IntToHex(Res, 8));
        break;
      }
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  cbIasValueType->ItemIndex = 0;
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerDisconnect(TObject *Sender,
      const TwclIrDAServerClientConnection *Client, const int Reason)
{
  lbEvents->Items->Add("Client disconnected: " +
    IntToHex((int)Client->Address, 8) +
    " reason: 0x" + IntToHex(Reason, 8));

  for (int i = 0; i < lvClients->Items->Count; i++)
    if (lvClients->Items->Item[i]->Data == (void*)Client)
    {
      lvClients->Items->Delete(i);
      break;
    }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerData(TObject *Sender,
      const TwclIrDAServerClientConnection *Client, const Pointer Data,
      const DWORD Size)
{
  if (Size > 0)
  {
    AnsiString Str;
    Str.SetLength(Size);
    CopyMemory(Str.c_str(), Data, Size);

    lbEvents->Items->Add("Received: " + Str + " from " +
      IntToHex((int)Client->Address, 8));
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerConnect(TObject *Sender,
      const TwclIrDAServerClientConnection *Client, const int Error)
{
  lbEvents->Items->Add("Client connected: " +
    IntToHex((int)Client->Address, 8) +
    " error: 0x" + IntToHex(Error, 8));

  if (Error == WCL_E_SUCCESS)
  {
    TListItem* Item = lvClients->Items->Add();
    Item->Caption = IntToHex((int)Client->Address, 8);
    Item->SubItems->Add(Client->Name);
    Item->Data = (void*)Client;
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerClosed(TObject *Sender,
      const int Reason)
{
  lbEvents->Items->Add("Server closed: " + IntToHex(Reason, 8));

  lvClients->Clear();

  edReadBuffer->Text = "";
  edWriteBuffer->Text = "";
}
//---------------------------------------------------------------------------

