//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclIrDAServers"
#pragma resource "*.dfm"

#ifdef _WIN64
#pragma comment(lib, "wclCommon.a")
#pragma comment(lib, "wclCommunication.a")
#pragma comment(lib, "wclIrDAFrameworkR.a")
#else
#pragma comment(lib, "wclCommon.lib")
#pragma comment(lib, "wclCommunication.lib")
#pragma comment(lib, "wclIrDAFrameworkR.lib")
#endif

TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbLog->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::Trace(String Msg)
{
  lbLog->Items->Add(Msg);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btListenClick(TObject *Sender)
{
  int Res = wclIrDAServer->Listen();
  if (Res != WCL_E_SUCCESS)
    ShowMessage("Error: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btCloseClick(TObject *Sender)
{
  int Res = wclIrDAServer->Close();
  if (Res != WCL_E_SUCCESS)
    ShowMessage("Error: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  // Simple try to close the server.
  wclIrDAServer->Close();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  wclIrDAServer->Service = "OBEX";
  wclIrDAServer->Mode = cmIrComm3Wire; // Required for OBEX connection!
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerListen(TObject *Sender)
{
  Trace("Server listening");        
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerClosed(TObject *Sender,
      const int Reason)
{
  Trace("Server closed. Reason:: 0x" + IntToHex(Reason, 8));        
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerDisconnect(TObject *Sender,
      const TwclIrDAServerClientConnection *Client, const int Reason)
{
  Trace("Client " + IntToHex((int)Client->Address, 8) + " disconnected. Reason: 0x" +
    IntToHex(Reason, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerDestroyProcessor(TObject *Sender,
      const TwclServerClientDataConnection *Connection)
{
  Trace("Data processor for client " +
    IntToHex((int)((TwclIrDAServerClientConnection*)Connection)->Address, 8) +
    " is destroing");
  Connection->Processor->Free();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerConnect(TObject *Sender,
      const TwclIrDAServerClientConnection *Client, const int Error)
{
  if (Error == WCL_E_SUCCESS)
    Trace("Client " + IntToHex((int)Client->Address, 8) + " connected");
  else
  {
    Trace("Client " + IntToHex((int)Client->Address, 8) +
      " connect failed: 0x" + IntToHex(Error, 8));
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAServerCreateProcessor(TObject *Sender,
      const TwclServerClientDataConnection *Connection)
{
  // Here we must create OBEX OPP server data processor.
  #if __BORLANDC__ >= 0x0650
	TwclObexOppServer* Proc = new TwclObexOppServer((TwclServerClientDataConnection*)Connection);
	#if __BORLANDC__ >= 0x0740
	  Proc->OnPutCompleted = OppClientPutCompleted;
	#else
	  Proc->OnPutCompleted = (TwclObexServerClientPutCompletedEvent)&OppClientPutCompleted;
	#endif
  #else
	TwclObexOppServer* Proc = new TwclObexOppServer(Connection);
	Proc->OnPutCompleted = OppClientPutCompleted;
  #endif
  Proc->OnConnect = OppClientConnect;
  Proc->OnDisconnected = OppClientDisconnected;
  Proc->OnPutBegin = OppClientPutBegin;
  Proc->OnPutProgress = OppClientPutProgress;
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::OppClientConnect(TObject* Sender, String Description)
{
  TwclObexOppServer* Proc = (TwclObexOppServer*)Sender;
  unsigned Address = ((TwclIrDAServerClientConnection*)Proc->Connection)->Address;
  bool Accept = MessageDlg("Client " + IntToHex((int)Address, 8) +
    " requsts for connection: " + Description + ". Accept?", mtConfirmation,
    TMsgDlgButtons() << mbYes << mbNo, 0) == mrYes;

  int Res;
  if (Accept)
    Res = Proc->Accept("Accepted by user");
  else
    Res = Proc->Reject("Rejected by user");

  if (Res != WCL_E_SUCCESS)
  {
    ShowMessage("Operation failed: 0x" + IntToHex(Res, 8));
    // Disconnect client!
    Proc->Connection->Disconnect();
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::OppClientDisconnected(TObject* Sender, int Reason,
  String Description)
{
  TwclObexOppServer* Proc = (TwclObexOppServer*)Sender;
  unsigned Address = ((TwclIrDAServerClientConnection*)Proc->Connection)->Address;

  Trace("Client " + IntToHex((int)Address, 8) +
    " closed OBEX session with reason: 0x" + IntToHex(Reason, 8) +
    ". Description: " + Description);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::OppClientPutBegin(TObject* Sender, String Name,
  String Description, String Mime, unsigned Length, bool& Accept)
{
  TwclObexOppServer* Proc = (TwclObexOppServer*)Sender;
  unsigned Address = ((TwclIrDAServerClientConnection*)Proc->Connection)->Address;
  String Str = "Client " + IntToHex((int)Address, 8) + " send file " + Name +
    ". Accept?";
  Accept = MessageDlg(Str, mtConfirmation, TMsgDlgButtons() << mbYes << mbNo, 0) == mrYes;

  if (Accept)
    FFileName = Name;
}
//---------------------------------------------------------------------------
#if __BORLANDC__ >= 0x0740
  void __fastcall TfmMain::OppClientPutCompleted(TObject* Sender,
	const int Error, TStream* const Stream, bool& Accept)
#else
  void __fastcall TfmMain::OppClientPutCompleted(TObject* Sender,
	const int Error, const TStream* Stream, bool& Accept)
#endif
{
  TwclObexOppServer* Proc = (TwclObexOppServer*)Sender;
  unsigned Address = ((TwclIrDAServerClientConnection*)Proc->Connection)->Address;
  if (Error == WCL_E_SUCCESS)
  {
    String Str = "Client " + IntToHex((int)Address, 8) + " completed sent file. Accept?";
    bool Accept = MessageDlg(Str, mtConfirmation, TMsgDlgButtons() << mbYes << mbNo, 0) == mrYes;

    if (Accept)
    {
      SaveDialog->FileName = FFileName;
      if (SaveDialog->Execute())
        ((TMemoryStream*)Stream)->SaveToFile(SaveDialog->FileName);
    }
  }
  else
    Trace("Send file error: 0x" + IntToHex(Error, 8));
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::OppClientPutProgress(TObject* Sender, unsigned Position,
  unsigned Length, bool& Continue)
{
  Trace("Sending file: " + IntToStr((int)Position) + " from " +
    IntToStr((int)Length));
  // Do not foget to set Continue flag!
  Continue = true;
}
//---------------------------------------------------------------------------
